@@ -36,11 +36,7 @@ NULL
   }
 }
 
-#' Helper for working with compound schemas
-#'
-#' This is similar to dbplyr::in_schema but has been tested across multiple
-#' database platforms. It only exists to work around some of the limitations
-#' of dbplyr::in_schema.
+#' Helper for working with compound schema
 #'
 #' @param schema A schema name as a character string
 #' @param table A table name as character string
@@ -50,14 +46,11 @@ NULL
 #' @return A DBI::Id that represents a qualified table and schema
 #' @export
 inSchema <- function(schema, table, dbms = NULL) {
-  # TODO deprecate this function after removing it from all tests
-  # lifecycle::deprecate_soft("1.4.1", "CDMConnector::inSchema()", "dbplyr::in_schema()")
   .inSchema(schema, table, dbms)
 }
 
 # internal function
 .inSchema <- function(schema, table, dbms = NULL) {
-  # lifecycle::deprecate_soft("1.4.1", "CDMConnector::inSchema()", "dbplyr::in_schema()")
   checkmate::assertCharacter(schema, min.len = 1, max.len = 3, null.ok = TRUE)
   checkmate::assertCharacter(table, len = 1, min.chars = 1)
   checkmate::assertCharacter(dbms, len = 1, null.ok = TRUE)
@@ -84,7 +77,7 @@ inSchema <- function(schema, table, dbms = NULL) {
     checkmate::assertCharacter(schema, min.len = 1, max.len = 2)
   }
 
-  if (isFALSE(dbms %in% c("snowflake", "sql server", "spark", "bigquery"))) {
+  if (isFALSE(dbms %in% c("snowflake", "sql server", "spark", "bigquery", "duckdb"))) {
     # only a few dbms support three part names
     checkmate::assertCharacter(schema, len = 1)
   }
@@ -104,6 +97,19 @@ inSchema <- function(schema, table, dbms = NULL) {
   return(out)
 }
 
+# Build a quoted qualified table name for use in raw SQL (e.g. INSERT INTO ...)
+# fullName: character table name or DBI::Id(schema=, table=) or DBI::Id(catalog=, schema=, table=)
+.qualifiedNameForSql <- function(con, fullName) {
+  if (is.character(fullName) && length(fullName) == 1) {
+    return(DBI::dbQuoteIdentifier(con, fullName))
+  }
+  if (inherits(fullName, "Id")) {
+    parts <- unlist(fullName@name)
+    return(paste(vapply(parts, function(p) DBI::dbQuoteIdentifier(con, p), character(1)), collapse = "."))
+  }
+  fullName
+}
+
 #' List tables in a schema
 #'
 #' DBI::dbListTables can be used to get all tables in a database but not always in a
@@ -137,6 +143,15 @@ listTables <- function(con, schema = NULL) {
     schema <- schema@name
   }
 
+  # Allow "catalog.schema" form (e.g. "cdm.main") for systems with multiple catalogs
+  if (length(schema) == 1 && is.character(schema) && stringr::str_detect(schema, "\\.")) {
+    if (stringr::str_count(schema, "\\.") != 1) {
+      rlang::abort("`schema` can only have one dot when using catalog.schema form.")
+    }
+    schema <- stringr::str_split(schema, "\\.")[[1]] %>%
+      purrr::set_names(c("catalog", "schema"))
+  }
+
   if ("prefix" %in% names(schema)) {
     prefix <- schema["prefix"]
     checkmate::assert_character(prefix, min.chars = 1, len = 1)
@@ -166,9 +181,9 @@ listTables <- function(con, schema = NULL) {
       return(temp_tables)
 
     } else if (dbms(con) == "snowflake") {
-      # return all tables including temp tables
-      return(DBI::dbGetQuery(con, "show terse tables;")$name)
-
+      # SHOW TERSE TABLES is limited to 10,000 rows; use information_schema to avoid that cap
+      sql <- "SELECT table_name AS name FROM information_schema.tables WHERE table_catalog = CURRENT_DATABASE() AND table_schema = CURRENT_SCHEMA()"
+      return(DBI::dbGetQuery(con, sql)$name)
     } else {
       return(DBI::dbListTables(con))
     }
@@ -189,7 +204,15 @@ listTables <- function(con, schema = NULL) {
   }
 
   if (methods::is(con, "duckdb_connection")) {
-    sql <- glue::glue_sql("select table_name from information_schema.tables where table_schema = {schema2[[1]]};", .con = con)
+    if (length(schema2) == 2) {
+      # Multiple catalogs (e.g. ATTACH): filter by table_catalog and table_schema
+      sql <- glue::glue_sql(
+        "SELECT table_name FROM information_schema.tables WHERE table_catalog = {schema2[[1]]} AND table_schema = {schema2[[2]]};",
+        .con = con
+      )
+    } else {
+      sql <- glue::glue_sql("SELECT table_name FROM information_schema.tables WHERE table_schema = {schema2[[1]]};", .con = con)
+    }
     out <- DBI::dbGetQuery(con, sql) %>% dplyr::pull(.data$table_name)
     return(process_prefix(out))
   }
@@ -377,6 +400,21 @@ dcCreateTable <- function(conn, name, fields) {
   }
 }
 
+# Convert Date/POSIXt to ISO character for Spark INSERT so ODBC doesn't produce malformed timestamps (e.g. 00-00-00).
+.formatDatesForSparkInsert <- function(table) {
+  dplyr::mutate(
+    table,
+    dplyr::across(
+      tidyselect::where(~ inherits(.x, "Date")),
+      ~ format(.x, "%Y-%m-%d")
+    ),
+    dplyr::across(
+      tidyselect::where(~ inherits(.x, "POSIXt")),
+      ~ format(.x, "%Y-%m-%d %H:%M:%S")
+    )
+  )
+}
+
 # build and execute the SQL query to insert data into the table
 # .dbInsertData <- function(conn, name, table) {
 #   columns <- colnames(table)
@@ -426,9 +464,11 @@ dcCreateTable <- function(conn, name, fields) {
 #' }
 computeDataHashByTable <- function(cdm) {
   overallStartTime <- Sys.time()
-  ensureInstalled("DatabaseConnector", "7")
   ensureInstalled("digest")
 
+  library(CDMConnector)
+  con <- DBI::dbConnect(duckdb::duckdb(), eunomiaDir())
+  cdm <- cdmFromCon(con, "main", "main")
   con <- cdmCon(cdm)
 
   cdmSchema <- attr(cdm, "cdm_schema")
@@ -457,8 +497,6 @@ computeDataHashByTable <- function(cdm) {
     "drug_strength" = "drug_concept_id",
     "cdm_source" = "NA")
 
-  tablesInCdmSchema <- tolower(listTables(con, schema = cdmSchema))
-
   out <- dplyr::tibble(
     cdm_name = character(),
     table_name = character(),
@@ -476,22 +514,24 @@ computeDataHashByTable <- function(cdm) {
     tableName <- names(cdmTables)[i]
     uniqueColumn <- cdmTables[i]
 
-    if (tableName %in% tablesInCdmSchema) {
+    if (tableName %in% names(cdm)) {
+      table_ref <- cdm[[tableName]]
       if (uniqueColumn != "NA") {
-        sql = "SELECT COUNT(*) AS n, COUNT(DISTINCT @unique_column) AS n_unique FROM @database_schema.@table_name;"
+        result <- table_ref |>
+          dplyr::summarise(
+            n = dplyr::n(),
+            n_unique = dplyr::n_distinct(.data[[uniqueColumn]])
+          ) |>
+          dplyr::collect()
       } else {
-        sql = "SELECT COUNT(*) AS n, -1 AS n_unique FROM @database_schema.@table_name;"
+        result <- table_ref |>
+          dplyr::summarise(
+            n = dplyr::n(),
+            n_unique = -1L
+          ) |>
+          dplyr::collect()
       }
 
-      result <- DatabaseConnector::renderTranslateQuerySql(
-        connection = con,
-        sql = sql,
-        database_schema = paste(cdmSchema, collapse = "."),
-        unique_column = uniqueColumn,
-        table_name = tableName,
-        warnOnMissingParameters = FALSE
-      )
-
       colnames(result) <- tolower(colnames(result))
 
       tableHash <- digest::digest(
@@ -516,7 +556,7 @@ computeDataHashByTable <- function(cdm) {
         table_row_count = -1L,
         unique_column = uniqueColumn,
         n_unique_values = -1L,
-        table_hash = "Table not found in CDM schema"
+        table_hash = "Table not found in CDM"
       )
     }
 
@@ -533,6 +573,206 @@ computeDataHashByTable <- function(cdm) {
   return(out)
 }
 
+#' Insert flattened CDM records as an aligned R comment in the active editor
+#'
+#' Flattens a CDM using [CDMConnector::cdmFlatten()], collects the data into R,
+#' optionally filters by one or more `person_id` values, and inserts an aligned,
+#' copy-pasteable comment block directly below the current cursor line in the
+#' active RStudio document.
+#'
+#' This is intended as a lightweight debugging and documentation helper when
+#' inspecting patient-level timelines (e.g. cohort inclusion, outcome validation,
+#' or study review notes).
+#'
+#' @param cdm A CDM reference object created with CDMConnector.
+#' @param personIds Optional numeric vector of `person_id` values to filter on.
+#'   If `NULL`, all persons in the flattened CDM are used (use with care).
+#'
+#' @return Invisibly returns the collected flattened CDM as a data.frame.
+#'   The primary side effect is insertion of commented text into the active
+#'   RStudio source editor.
+#'
+#' @details
+#' The inserted output is formatted as aligned R comments:
+#'
+#' \preformatted{
+#' # person_id | observation_concept_id | start_date | end_date   | domain
+#' # 12        | 2211751                 | 2021-01-13 | 2021-01-13 | procedure_occurrence
+#' }
+#'
+#' The function requires an interactive RStudio session and will error if
+#' `rstudioapi` is not available.
+#'
+#' @seealso
+#' [CDMConnector::cdmFlatten()]
+#'
+#' @examples
+#' \dontrun{
+#' # Insert patient timeline directly into your script
+#' cdmCommentContents(cdm, 12)
+#'
+#' # Insert multiple patients
+#' cdmCommentContents(cdm, c(12, 22))
+#' }
+#'
+#' @keywords internal
+#' @noRd
+.cdm_comment_interactive <- function() base::interactive()
 
+#' @keywords internal
+#' @noRd
+.cdm_comment_require_rstudioapi <- function() requireNamespace("rstudioapi", quietly = TRUE)
+
+#' @keywords internal
+#' @noRd
+.cdm_comment_rstudio_available <- function() rstudioapi::isAvailable()
+
+#' @keywords internal
+#' @noRd
+.cdm_comment_get_context <- function() rstudioapi::getActiveDocumentContext()
+
+#' @keywords internal
+#' @noRd
+.cdm_comment_doc_position <- function(row, column) rstudioapi::document_position(row, column)
+
+#' @keywords internal
+#' @noRd
+.cdm_comment_insert_text <- function(location, text) rstudioapi::insertText(location, text)
+
+#' @keywords internal
+#' @noRd
+.cdm_comment_flatten <- function(cdm) CDMConnector::cdmFlatten(cdm)
+
+#' @keywords internal
+#' @noRd
+.cdm_comment_collect <- function(x) dplyr::collect(x)
+
+#' Insert Patient CDM Contents as Aligned Comments in RStudio
+#'
+#' This function retrieves the longitudinal event table for one or more persons
+#' in a CDM object and inserts it as a nicely formatted, R-style comment block
+#' directly into your active RStudio document. This is particularly useful for
+#' documenting reproducible test cases or examples by showing relevant CDM
+#' contents inline in test scripts or analysis code.
+#'
+#' Each row of patient data will be aligned in columns as a commented table, making
+#' it easy to copy, review, and maintain sample data expectations in documentation
+#' or test suites.
+#'
+#' @param cdm A CDMConnector cdm_reference object.
+#' @param personIds Optional numeric vector of person IDs to filter the rows to include.
+#'   If NULL (default), includes all persons in the cdm.
+#'
+#' @details
+#' Requires an interactive RStudio session with the \code{rstudioapi} package available.
+#' The function utilizes \code{CDMConnector::cdmFlatten()} to extract a longitudinal view,
+#' and writes the commented results directly below the cursor in the active RStudio document.
+#'
+#' This workflow is especially helpful when documenting expected patient timelines for use in
+#' testthat or other test scripts, or when sharing reproducible CDM content for instructional
+#' examples.
+#'
+#' @seealso \code{\link[CDMConnector]{cdmFlatten}}
+#' @examples
+#' \dontrun{
+#' library(CDMConnector)
+#' con <- DBI::dbConnect(duckdb::duckdb(), eunomiaDir())
+#' cdm <- cdmFromCon(con, "main", "main")
+#' cdmCommentContents(cdm, personIds = 6)
+#' # person_id | observation_concept_id | start_date | end_date   | type_concept_id...
+#' # 6         | 40213296               | 2006-01-10 | 2006-01-10 | 581452         ...
+#' # 6         | 40213227               | 2006-01-10 | 2006-01-10 | 581452         ...
+#' # 6         | 1118084                | 2005-07-13 | 2005-07-13 | 38000177       ...
+#' # 6         | 80180                  | 2005-07-13 | NA         | 32020          ...
+#' cdmDisconnect(cdm)
+#' }
+#'
+#' @export
+cdmCommentContents <- function(cdm, personIds = NULL) {
+  # This function only makes sense in interactive use.
+  if (!.cdm_comment_interactive()) return(invisible(NULL))
+
+  if (!.cdm_comment_require_rstudioapi()) {
+    stop("Package 'rstudioapi' is required.", call. = FALSE)
+  }
+  if (!.cdm_comment_rstudio_available()) {
+    stop("RStudio is required (rstudioapi is not available in this session).", call. = FALSE)
+  }
+
+  if (!is.null(personIds) && !is.numeric(personIds)) {
+    stop("`personIds` must be a numeric vector (or NULL).", call. = FALSE)
+  }
+
+  checkmate::assertClass(cdm, "cdm_reference")
+  checkmate::assertTRUE("person" %in% names(cdm))
+
+  if (is.null(personIds)) {
+    nPersons <- cdm$person %>% dplyr::ungroup() %>% dplyr::tally() %>% dplyr::pull("n")
+    if (nPersons > 1000) {
+      cli::cli_abort("Your cdm has {nPersons} persons in it. Use the `personIds` argument to subset to just a few persons for printing.")
+    }
+  }
+
+  # 1) Run the pipeline (cdmFlatten -> collect -> optional filter -> arrange)
+  flat <- .cdm_comment_flatten(cdm) |>
+    .cdm_comment_collect()
+
+  if (!is.null(personIds)) {
+    flat <- dplyr::filter(flat, .data$person_id %in% personIds)
+  }
+
+  flat <- dplyr::arrange(
+    flat,
+    .data$person_id,
+    dplyr::desc(.data$start_date),
+    dplyr::desc(.data$end_date)
+  ) |>
+  head(1000) # limit comment to 1000 lines
+
+  # 2) Convert to aligned comment text (like your first format)
+  df_chr <- as.data.frame(lapply(flat, as.character), stringsAsFactors = FALSE)
+
+  widths <- vapply(
+    names(df_chr),
+    function(col) max(nchar(c(col, df_chr[[col]])), na.rm = TRUE),
+    integer(1)
+  )
+
+  pad <- function(x, w) sprintf(paste0("%-", w, "s"), x)
+
+  lines <- character()
+  header <- paste(mapply(pad, names(df_chr), widths), collapse = " | ")
+  lines <- c(lines, paste0("# ", header))
+
+  for (i in seq_len(nrow(df_chr))) {
+    row <- paste(mapply(pad, df_chr[i, ], widths), collapse = " | ")
+    lines <- c(lines, paste0("# ", row))
+  }
+
+  # 3) Insert directly below the current line in the active RStudio document
+  ctx <- .cdm_comment_get_context()
+  contents <- ctx$contents
+
+  # RStudio rows are 1-based
+  cursor_row <- ctx$selection[[1]]$range$start[["row"]]
+
+  # Find the most recent call line above (or at) the cursor
+  call_row <- cursor_row
+  while (call_row >= 1 && !grepl("\\bcdmCommentContents\\s*\\(", contents[[call_row]])) {
+    call_row <- call_row - 1
+  }
+
+  if (call_row < 1) {
+    stop("Couldn't find a line containing `cdmCommentContents(` above the cursor.", call. = FALSE)
+  }
+
+  # Insert *immediately below* the call line
+  .cdm_comment_insert_text(
+    location = .cdm_comment_doc_position(call_row + 1, 1),
+    text = paste0(paste(lines, collapse = "\n"), "\n")
+  )
+
+  invisible(flat)
+}
 
 
